// $Id: TestableSharingBufferReaderProvider.java 19474 2010-05-12 11:03:46Z stliu $
package org.hibernate.search.test.reader.functionality;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.MultiReader;
import org.apache.lucene.store.Directory;
import org.hibernate.search.SearchException;
import org.hibernate.search.engine.SearchFactoryImplementor;
import org.hibernate.search.reader.ReaderProviderHelper;
import org.hibernate.search.reader.SharingBufferReaderProvider;
import org.hibernate.search.store.DirectoryProvider;

/**
 * @author Sanne Grinovero
 */
public class TestableSharingBufferReaderProvider extends SharingBufferReaderProvider {

	private static final int NUM_DIRECTORY_PROVIDERS = 4;
	private final Vector<MockIndexReader> createdReadersHistory = new Vector<MockIndexReader>( 500 );
	final Map<Directory, TestManipulatorPerDP> manipulators = new ConcurrentHashMap<Directory, TestManipulatorPerDP>();
	final List<DirectoryProvider> directoryProviders = Collections.synchronizedList(new ArrayList<DirectoryProvider>());
	
	public TestableSharingBufferReaderProvider() {
		for ( int i = 0; i < NUM_DIRECTORY_PROVIDERS; i++ ) {
			TestManipulatorPerDP tm = new TestManipulatorPerDP( i );
			manipulators.put( tm.dp.getDirectory(), tm );
			directoryProviders.add( tm.dp );
		}
	}

	public boolean isReaderCurrent(MockIndexReader reader) {
		//avoid usage of allReaders or test would be useless
		for ( PerDirectoryLatestReader latest : currentReaders.values() ) {
			IndexReader latestReader = latest.current.reader;
			if ( latestReader == reader ) {
				return true;
			}
		}
		return false;
	}

	@Override
	protected IndexReader readerFactory(Directory directory) {
		TestManipulatorPerDP manipulatorPerDP = manipulators.get( directory );
		if ( !manipulatorPerDP.isReaderCreated.compareAndSet( false, true ) ) {
			throw new IllegalStateException( "IndexReader1 created twice" );
		}
		else {
			return new MockIndexReader( manipulatorPerDP.isIndexReaderCurrent, createdReadersHistory );
		}
	}

	@Override
	public void initialize(Properties props, SearchFactoryImplementor searchFactoryImplementor) {
		try {
			for ( Directory directory : manipulators.keySet() ) {
				currentReaders.put( directory, new PerDirectoryLatestReader( directory ) );
			}
		}
		catch ( IOException e ) {
			throw new SearchException( "Unable to open Lucene IndexReader", e );
		}
	}

	public boolean areAllOldReferencesGone() {
		int numReferencesReaders = allReaders.size();
		int numExpectedActiveReaders = manipulators.size();
		return numReferencesReaders == numExpectedActiveReaders;
	}

	public List<MockIndexReader> getCreatedIndexReaders() {
		return createdReadersHistory;
	}

	public MockIndexReader getCurrentMockReaderPerDP(DirectoryProvider dp) {
		IndexReader[] indexReaders = ReaderProviderHelper.getSubReadersFromMultiReader(
				( MultiReader ) super.openReader(
						new DirectoryProvider[] { dp }
				)
		);
		if ( indexReaders.length != 1 ) {
			throw new IllegalStateException( "Expecting one reader" );
		}
		return ( MockIndexReader ) indexReaders[0];
	}

	

}
